<?php

/**
 * Copyright Souke (2018)
 *
 * contact@souke.fr
 *
 * Ce logiciel est un programme informatique servant à aider les producteurs
 * à distribuer leur production en circuits courts.
 *
 * Ce logiciel est régi par la licence CeCILL soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL telle que diffusée par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 *
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 *
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement,
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.
 *
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez
 * pris connaissance de la licence CeCILL, et que vous en avez accepté les
 * termes.
 */

namespace backend\controllers;

use common\helpers\GlobalParam;
use domain\Producer\ProducerPriceRange\ProducerPriceRange;
use domain\Producer\Producer\Producer;
use Yii;
use yii\data\ActiveDataProvider;
use yii\filters\AccessControl;
use yii\filters\VerbFilter;
use yii\web\NotFoundHttpException;
use yii\web\UploadedFile;

/**
 * UserController implements the CRUD actions for User model.
 */
class ProducerController extends BackendController
{
    public $enableCsrfValidation = false;

    public function behaviors()
    {
        return [
            'verbs' => [
                'class' => VerbFilter::class,
                'actions' => [
                    'delete' => ['post'],
                ],
            ],
            'access' => [
                'class' => AccessControl::class,
                'rules' => [
                    [
                        'allow' => true,
                        'roles' => ['@'],
                        'matchCallback' => function ($rule, $action) {
                            return $this->getUserModule()
                                ->getAuthorizationChecker()
                                ->isGrantedAsProducer($this->getUserCurrent());
                        }
                    ]
                ],
            ],
        ];
    }

    /**
     * Modifie un producteur.
     */
    public function actionUpdate()
    {
        $producerModule = $this->getProducerModule();
        $producerBuilder = $producerModule->getBuilder();
        $request = Yii::$app->request;
        $model = $this->findModel(GlobalParam::getCurrentProducerId());

        $logoFilenameOld = $model->logo;
        $photoFilenameOld = $model->photo;
        $documentImageBottomFilenameOld = $model->document_image_bottom;
        $producerBuilder->initOptionsDatesDisplay($model);

        if ($model->load(\Yii::$app->request->post())) {

            $model->logoFile = UploadedFile::getInstance($model, 'logoFile');
            $model->photoFile = UploadedFile::getInstance($model, 'photoFile');
            $model->document_image_bottomFile = UploadedFile::getInstance($model, 'document_image_bottomFile');

            if($model->validate()) {

                $producerBuilder->initOptionsDatesBeforeSave($model);
                $producerBuilder->processUploadImage($model, 'logo', $logoFilenameOld, $request->post('delete_logo', 0));
                $producerBuilder->processUploadImage($model, 'photo', $photoFilenameOld, $request->post('delete_photo', 0));
                $producerBuilder->processUploadImage($model, 'document_image_bottom', $documentImageBottomFilenameOld, $request->post('delete_document_image_bottom', 0));
                $producerBuilder->savePrivateKeysStripe($model);
                $model->save();

                $this->setFlash('success', 'Paramètres mis à jour.');
                return $this->redirect(['update', 'id' => $model->id, 'edit_ok' => true]);
            }
        }

        if ($model->load(\Yii::$app->request->post())) {
            $this->setFlash('error', 'Le formulaire comporte des erreurs.');
        }

        return $this->render('update', [
            'model' => $model,
        ]);
    }

    /**
     * Affiche le formulaire permettant au producteur de définir le montant
     * de son abonnement.
     */
    public function actionBilling()
    {
        $producer = Producer::findOne(GlobalParam::getCurrentProducerId());

        if ($producer->load(\Yii::$app->request->post())) {
            $producer->save();

            if (!is_null($producer->free_price)) {
                $alertFreeprice = true;
            }
        }

        $dataProviderPrices = new ActiveDataProvider([
            'query' => ProducerPriceRange::find()
                ->orderBy('id ASC'),
            'pagination' => [
                'pageSize' => 100,
            ],
        ]);

        return $this->render('billing', [
            'paidFeaturesArray' => $this->getFeatureModule()->getRepository()->findPaidFeatures(),
            'dataProviderPrices' => $dataProviderPrices,
            'producer' => $producer,
            'alertFreePrice' => (isset($alertFreeprice)) ? true : false
        ]);
    }

    /**
     * Recherche un établissement via son ID.
     */
    protected function findModel(int $id)
    {
        if (($model = Producer::findOne($id)) !== null) {
            return $model;
        } else {
            throw new NotFoundHttpException('The requested page does not exist.');
        }
    }

    public function actionUpdateOpendistribVersion()
    {
        $producerModule = $this->getProducerModule();
        $producerModule->updateOpendistribVersion($this->getProducerCurrent());

        return $this->redirect(\Yii::$app->request->referrer);
    }

    public function actionUpdateAlertMessageReadAt()
    {
        $this->getProducerModule()->getManager()->readAlertMessage($this->getProducerCurrent());
        return $this->redirect(\Yii::$app->request->referrer);
    }

    public function actionPagesize()
    {
        $producerCurrent = $this->getProducerCurrent();
        $request = Yii::$app->request;

        if($request->isPost) {
            $entity = $request->post('entity');
            $pageSize = $request->post('pagesize');

            $this->getProducerModule()->getProducerPageSizer()
                ->updatePagesize($producerCurrent, $entity, $pageSize);
        }
        
        $referer = strtok(Yii::$app->request->referrer, "?");
        return $this->redirect($referer);
    }
}
